namespace USPS
{
	using System;
	using System.Data;
	using System.Configuration;
	using System.Web;
	using System.Web.Security;
	using System.Web.UI;
	using System.Web.UI.WebControls;
	using System.Web.UI.WebControls.WebParts;
	using System.Web.UI.HtmlControls;
	using System.Net;
	using System.IO;
	using System.Text;
	using System.Xml;

	/// <summary>
	/// Summary description for USPSRates
	/// </summary>



	public class USPSRates
	{

		// Declare required fields
		private string apiModule;
		public int packageId;
		private string userId;
		private string password;
		public string shipService;
		public string zipOrigination;
		public string zipDestination;
		public int pounds;
		public int ounces;
		public string containerType;
		public string packageSize;
		public bool machinable;
		private string url;
		public string country;
		public bool IsInternational;

		// Initialize field values
		public USPSRates()
		{
			apiModule = string.Empty;
			packageId = 0;
			userId = string.Empty;
			password = string.Empty;
			shipService = string.Empty;
			zipOrigination = string.Empty;
			zipDestination = string.Empty;
			pounds = 0;
			ounces = 0;
			containerType = string.Empty;
			packageSize = string.Empty;
			machinable = false;
			url = string.Empty;
			country = string.Empty;
			IsInternational = false;
		}

		// Send the request to USPS
		public string RequestUSPSRates()
		{
			try
			{
				InitiateService();
				string _xmlRequestString;
				string _xmlResponseString;

				if (IsInternational)
				{
					_xmlRequestString = BuildInternationalXMLRequest();
				}
				else
				{
					_xmlRequestString = BuildDomesticXMLRequest();
				}

				Uri USPSUri = new Uri(url + "API=" + apiModule + "&XML=" + _xmlRequestString);
				// Create a new request to the above mentioned URL.    
				WebRequest myWebRequest = WebRequest.Create(USPSUri);
				// Assign the response object of 'WebRequest' to a 'WebResponse' variable.
				WebResponse myWebResponse = myWebRequest.GetResponse();

				StreamReader sr = new StreamReader(myWebResponse.GetResponseStream());
				_xmlResponseString = sr.ReadToEnd();
				return _xmlResponseString;
			}
			catch
			{
				return "E";
			}
		}

		//Initialize service parameters
		private void InitiateService()
		{
			string _serivceMode = ConfigurationManager.AppSettings["ServiceMode"];
			password = ConfigurationManager.AppSettings["USPSPassword"];
			userId = ConfigurationManager.AppSettings["USPSKey"];
			if (IsInternational)
			{
				apiModule = "IntlRate";
			}
			else
			{
				apiModule = "RateV2";
			}

			if (_serivceMode == "Test")
			{
				url = ConfigurationManager.AppSettings["USPSTestURL"];
			}
			else
			{
				url = ConfigurationManager.AppSettings["USPSProdURL"];
			}
		}

		// Construct the request xml string
		private string BuildDomesticXMLRequest()
		{
			StringWriter sw = new StringWriter();
			XmlTextWriter xmlWriter = new XmlTextWriter(sw);
			xmlWriter.WriteStartDocument();
			xmlWriter.WriteStartElement("RateV2Request");
			xmlWriter.WriteAttributeString("USERID", userId);
			xmlWriter.WriteAttributeString("PASSWORD", password);
			xmlWriter.WriteStartElement("Package");
			xmlWriter.WriteAttributeString("ID", packageId.ToString());
			xmlWriter.WriteElementString("Service", shipService);
			xmlWriter.WriteElementString("ZipOrigination", zipOrigination);
			xmlWriter.WriteElementString("ZipDestination", zipDestination);
			xmlWriter.WriteElementString("Pounds", pounds.ToString());
			xmlWriter.WriteElementString("Ounces", ounces.ToString());
			xmlWriter.WriteElementString("Container", containerType.ToString());
			xmlWriter.WriteElementString("Size", packageSize);
			if (shipService == "PARCEL POST" || shipService == "All")
			{
				if (machinable)
				{
					xmlWriter.WriteElementString("Machinable", "true");
				}
				else
				{
					xmlWriter.WriteElementString("Machinable", "false");
				}
			}
			xmlWriter.WriteEndElement();
			xmlWriter.WriteEndElement();
			return sw.ToString();
		}

		private string BuildInternationalXMLRequest()
		{
			StringWriter sw = new StringWriter();
			XmlTextWriter xmlWriter = new XmlTextWriter(sw);
			xmlWriter.WriteStartDocument();
			xmlWriter.WriteStartElement("IntlRateRequest");
			xmlWriter.WriteAttributeString("USERID", userId);
			xmlWriter.WriteAttributeString("PASSWORD", password);
			xmlWriter.WriteStartElement("Package");
			xmlWriter.WriteAttributeString("ID", packageId.ToString());
			xmlWriter.WriteElementString("Pounds", pounds.ToString());
			xmlWriter.WriteElementString("Ounces", ounces.ToString());
			xmlWriter.WriteElementString("MailType", containerType.ToString());
			xmlWriter.WriteElementString("Country", country.ToString());
			xmlWriter.WriteEndElement();
			xmlWriter.WriteEndElement();
			return sw.ToString();
		}
	}
}
